## helper functions

create.tree.model <- function(meta)
{
    filter.meta <- as.data.frame(cbind(filter = rep(TRUE, length.out = nrow(meta)), meta))
    pmodel <- rGtkDataFrame(filter.meta)
    filter.model <- gtkTreeModelFilterNew(pmodel)
    filter.model$setVisibleColumn(0)
    ##  sort.model <- gtkTreeModelSort(filter.model)
    ##  sort.model
    filter.model
}

create.tree.view <-
    function(pmodel, meta)
{
    tree.view <- gtkTreeView(pmodel)
    sapply(seq_len(ncol(meta)),
           function(w){
               renderer <- gtkCellRendererText()
               column <- gtkTreeViewColumn(colnames(meta)[w], renderer, text = w)
               column$setSortColumnId(w)
               column$setCellDataFunc(renderer,
                                      function(column, renderer, pmodel, iter)
                                  {
                                      q <- pmodel$getPath(iter)$getIndices()[[1]] + 1
                                      renderer["text"] <- as.character(meta[q, w])
                                  })
               tree.view$appendColumn(column)
           })
    tree.view$setHeadersClickable(TRUE)
    if(is.null(gtkCheckVersion(2, 10, 0)))
        tree.view$setGridLines("both")
    tree.view
}



## update the "Information" pane
updateInformation <- function()
{
    df <- activeFlowset()
    if (!isAnyActive())
    {
        populateDataView("No dataset active.")
    }
    else 
    {
        info1 <- paste("Number of Samples:", length(df))
        m.name <- paste(colnames(df), collapse = " , ")
        info2 <- paste("Channel Names :", m.name)
        populateDataView(c(info1, info2))
    }
}

## populate the phenoData panel after the data set has been changed
updatePdata <- function(action)
{
    pdata <- pData(activeFlowset())
    name <- "Annotation"
    iget("notebook")$removePage(1)
    if (nrow(pdata) > 0)
    {
        meta <- cbind(rownames = rownames(pdata), pdata)
        ## WAS: meta <- cbind(rownames = rownames(iget("meta")), iget("meta"))

        pmodel <- create.tree.model(meta)
        ptree.view <- create.tree.view(pmodel, meta)
        ptree.view$getSelection()$setMode("multiple")
        ptree.view$setHeadersClickable(TRUE)
        selected <- ptree.view$getSelection()
        ## gSignalConnect(selected, "changed", recordSelection, ptree.view)
        vbox <- gtkVBox(FALSE, 5)
        scrolled_window <- gtkScrolledWindow()
        scrolled_window$add(ptree.view)
        vbox$packStart(scrolled_window, TRUE, TRUE, 0)
        iset("phenodata.tree.view", ptree.view) ## used to retrieve selection

        iget("notebook")$insertPage(vbox, gtkLabel(name), 1)
        
        gSignalConnect(ptree.view, "button-press-event",
                       button.cb, c(ptree.view, action, "pheno"))
    }
}


## getter and setter for the current notebook pages
getCurrentPage <- function()
{
    iget("notebook")$getCurrentPage()
}

setCurrentPage <- function(n)
{
    iget("notebook")$setCurrentPage(n)
}


## event handler when the data set is changed

dataChoice <- function(selected, info)
{
    cpage <- getCurrentPage()
    on.exit(setCurrentPage(cpage))
    inds <- getSelectedIndices(iget("dataSetTreeView"))
    if (length(inds) == 0) {
        iset("active.workflow", "")
    }
    else
    {
        iset("active.workflow", names(iget("workflow.list"))[inds[1]])
    }
    iset("df", activeFlowset())
    updateInformation() 
    updatePdata(action) 
      
 }



