

## a dialog for common graphics choices, like sample names,
## x-variable(s), etc.

ask.graphics.details <-
    function(action, window = NULL,
             title = graphics.types[iget("graphics.choice")],
             allow.multiple.samples = TRUE,
             channel.selections = c("X Channel", "Y Channel"),
             allow.multiple.channels = FALSE)
    ## one column for each 'channel.selections'
{
    ## FIXME: put this in history somehow
    channels <- colnames(activeFlowset())
    sample.names <- sampleNames(activeFlowset())
    if (length(sample.names) == 1 && length(channel.selections) == 0) # e.g., splom with a one-sample flowSet
    {
        return(list(ok = TRUE, selected.samples = sample.names))
    }
    allow.multiple.channels <- rep(allow.multiple.channels, length.out = length(channel.selections))
    dialog <- gtkDialogNewWithButtons(title, window, "destroy-with-parent", 
                                      "gtk-ok", GtkResponseType["ok"],
                                      "gtk-cancel", GtkResponseType["cancel"])
    dialog$setResponseSensitive(GtkResponseType["ok"], FALSE)
    updateOKButton <- function(sel, udata = NULL)
    {
        make.sensitive <- TRUE
        if (length(sample.names) > 1 && length(getSelectedIndices(fview)) == 0)
            make.sensitive <- FALSE
       # for (cv in channel.views)
       #     if (length(getSelectedIndices(cv)) == 0)
       #         make.sensitive <- FALSE
        dialog$setResponseSensitive(GtkResponseType["ok"], make.sensitive)
    }
    main.box <- gtkHBox(FALSE, 10)
    dialog[["vbox"]]$packStart(main.box, FALSE, FALSE, 0)
    if (length(sample.names) != 1)
    {
        ## for the view of sample names
        outframe <- gtkFrame("")
        fhbox <- gtkHBox(FALSE, 5)
        fhbox$setSizeRequest(250, 350)
        outframe$add(fhbox)
        main.box$packStart(outframe, TRUE, TRUE, 0) 
        
        frdf <- rGtkDataFrame(data.frame(cols=sample.names))
        fview <- gtkTreeView(frdf)
        gSignalConnect(fview$getSelection(), "changed", updateOKButton, NULL)

        fcol <- gtkTreeViewColumn("Samples", gtkCellRendererText(), text = 0)
        fview$insertColumn(fcol, 0)
        fview$getSelection()$setMode(if (allow.multiple.samples) "multiple" else "single")
        if (iget("use.graphics.subset")) # when called from the context menu in the phenodata browser
        {
            iset("use.graphics.subset", FALSE)
            setSelectedIndices(fview, activePhenoSubset())
        }
        sw <- gtkScrolledWindowNew(NULL, NULL)
        sw$setShadowType("etched-in")
        sw$setPolicy("automatic", "automatic")
        sw$add(fview)
        fhbox$packStart(sw, TRUE, TRUE, 0)
    }

    channelSelectWidget <- function(i)
    {
        outchannel <- gtkFrame("")
        chbox <- gtkHBox(FALSE, 5)
        chbox$setSizeRequest(250, 350)
        outchannel$add(chbox)
        main.box$packStart(outchannel, TRUE, TRUE, 0)
        ## The list of available channels
        rdf <- rGtkDataFrame(data.frame(cols=channels))
        view <- gtkTreeView(rdf)
        col <- gtkTreeViewColumn(channel.selections[i], gtkCellRendererText(), text = 0)
        view$insertColumn(col, 0)
        view$getSelection()$setMode(if (allow.multiple.channels[i]) "multiple" else "single")
        sw2 <- gtkScrolledWindowNew(NULL, NULL)
        sw2$setShadowType("etched-in")
        sw2$setPolicy("automatic", "automatic")
        sw2$add(view)
        chbox$packStart(sw2, TRUE, TRUE, 0)
        gSignalConnect(view$getSelection(), "changed", updateOKButton, NULL)
        view
    }
    channel.views <- lapply(seq_along(channel.selections), channelSelectWidget)
    names(channel.views) <- channel.selections
    diag.run <- dialog$run()
    if (diag.run == GtkResponseType["ok"])
    {
        ans <- list(ok = TRUE)
        ans$selected.samples <-
            if (length(sample.names) == 1) sample.names
            else sample.names[getSelectedIndices(fview)]
        ans$selected.channels <- lapply(channel.views, function(v) channels[getSelectedIndices(v)])
        ans <-
            within(ans,
               {
                   if (length(sample.names) == 0) ok <- FALSE
                   for (i in selected.channels)
                       if (length(i) == 0) ok <- FALSE
               })
    }
    else ## if (diag.run %in% GtkResponseType[c("cancel", "delete-event")])
    {
        ans <- list(ok = FALSE)
    }
    dialog$destroy()
    ans
}


## menu with choice of graphics (similar interface in iflow.R, as menu actions)

graphics.menu <- function(action, window)
{
    if (!isAnyActive()) 
    {
        show.message("No active dataset")
        return()
    }
    ## A dialogue with radio buttons to select plot type
    dialog <- gtkDialogNewWithButtons("Graphics", window, "destroy-with-parent", 
                                      "gtk-ok", GtkResponseType["ok"], "gtk-cancel",
                                      GtkResponseType["cancel"])
    label <- gtkLabel("Please select plot type")
    dialog[["vbox"]]$packStart(label, TRUE, TRUE, 10)
    plot.frm <- gtkFrame("Plots")
    dialog[["vbox"]]$packStart(plot.frm, FALSE, FALSE, 10)
    radio_buttons <- NULL
    vbox.plot <- gtkVBox(FALSE, 0)
    plot.frm$add(vbox.plot)
    for (choice in seq_along(graphics.types))
    {
        button <- gtkRadioButton(radio_buttons, graphics.types[choice])
        vbox.plot$add(button)
        radio_buttons <- c(radio_buttons, button)
    }
    ## evaluate the selection
    if(dialog$run() == GtkResponseType["ok"])
    {
        buttons <- button$getGroup()
        active <- rev(names(graphics.types))[ which(sapply(buttons, gtkToggleButtonGetActive)) ]
        iset("graphics.choice", active)
        dialog$destroy()
        switch(active,
               contour = contour.menu(action, window),
               ecdf = ecdf.menu(action, window),
               histogram = histogram.menu(action, window),
               parallel = parallel.menu(action, window), 
               qqplot = qqmath.menu(action, window),
               splom = splom.menu(action, window),
               smooth.xy = xyplot.menu(action, window),
               dot.plot = dotplot.menu(action, window), 
               density = densityplot.menu(action, window),
               timeline = timeLine.menu(action, window),
               workflow = workflow.menu(action, window),
               stop("unknown plot type"))
    }
   dialog$destroy()
}                      


histogram.menu <- function(action, window)
{
  graphics.choice <- "histogram"
  iset("graphics.choice", graphics.choice)
  gdetails <- ask.graphics.details(action, window, title = "Histogram", allow.multiple.samples = FALSE, channel.selections = c("X Channel", "Y Channel"))
    if (gdetails$ok)
    {
        iset("selected.graphics.subset", gdetails$selected.samples)
        iset("value.x", gdetails$selected.channels[[1]])
        iset("value.y", gdetails$selected.channels[[2]])
        openGraphics(mode = 1)
    }
}  

contour.menu <- function(action, window)
{
  graphics.choice <- "contour"
  iset("graphics.choice", graphics.choice)
  ## IT HAS TO ALLOW TO CHOOSE MULTIPLE SAMPLES
  gdetails <- ask.graphics.details(action, window, allow.multiple.samples = FALSE, channel.selections = c("X Channel", "Y Channel"))
    if (gdetails$ok)
    {
        iset("selected.graphics.subset", gdetails$selected.samples)
        iset("value.x", gdetails$selected.channels[[1]])
        iset("value.y", gdetails$selected.channels[[2]])
        openGraphics(mode = 1)
    }
}  

xyplot.menu <- function(action, window)
{
    graphics.choice <- "smooth.xy"
   iset("graphics.choice", graphics.choice)
    gdetails <- ask.graphics.details(action, window, channel.selections = c("X Channel", "Y Channel"))
    if (gdetails$ok)
    {
        iset("selected.graphics.subset", gdetails$selected.samples)
        iset("value.x", gdetails$selected.channels[[1]])
        iset("value.y", gdetails$selected.channels[[2]])
        openGraphics(mode = 1)
    }
}

dotplot.menu <- function(action, window)
{
    graphics.choice <- "dot.plot"
    iset("graphics.choice", graphics.choice)
    gdetails <- ask.graphics.details(action, window, channel.selections = c("X Channel", "Y Channel"))
    if (gdetails$ok)
    {
        iset("selected.graphics.subset", gdetails$selected.samples)
        iset("value.x", gdetails$selected.channels[[1]])
        iset("value.y", gdetails$selected.channels[[2]])
        openGraphics(mode = 1)
    }
}

densityplot.menu <- function(action, window)
{
    graphics.choice <- "density"
    iset("graphics.choice", graphics.choice)
    gdetails <- ask.graphics.details(action, window, channel.selections = "Channels", allow.multiple.channels = TRUE)
    if (gdetails$ok)
    {
        iset("selected.graphics.subset", gdetails$selected.samples)
        iset("value.x", gdetails$selected.channels[[1]])
        iset("value.y", NULL)
        openGraphics(mode = 1)
    }
}


splom.menu <- function(action, window)
{   
    graphics.choice <- "splom"
    iset("graphics.choice", graphics.choice) 
    gdetails <- ask.graphics.details(action, window, allow.multiple.samples = FALSE,
                                     channel.selections = "Channels", allow.multiple.channels = TRUE)
    if (gdetails$ok)
    {
        iset("selected.graphics.subset", gdetails$selected.samples)
        iset("value.x", gdetails$selected.channels[[1]])
        iset("value.y", NULL)
        openGraphics(mode = 1)
    }
}

parallel.menu <- function(action, window)
{
     graphics.choice <- "parallel"
    iset("graphics.choice", graphics.choice)
    gdetails <- ask.graphics.details(action, window, allow.multiple.samples = FALSE,
                                     channel.selections = "Channels", allow.multiple.channels = TRUE)
    if (gdetails$ok)
    {
        iset("selected.graphics.subset", gdetails$selected.samples)
        iset("value.x", gdetails$selected.channels[[1]])
        iset("value.y", NULL)
        openGraphics(mode = 1)
    }
}


timeLine.menu <- function(action, window)
{
    graphics.choice <- "timeline"
    iset("graphics.choice", graphics.choice)
    gdetails <- ask.graphics.details(action, window, channel.selections = "Channel")
    if (gdetails$ok)
    {
        iset("selected.graphics.subset", gdetails$selected.samples)
        iset("value.x", gdetails$selected.channels[[1]])
        iset("value.y", NULL)
        openGraphics(mode = 1)
    }
}


qqmath.menu <- function(action, window)
{
    graphics.choice <- "qqplot"
    iset("graphics.choice", graphics.choice)
    gdetails <- ask.graphics.details(action, window, allow.multiple.samples = TRUE, channel.selections = "Channel", allow.multiple.channels = FALSE)
    if (gdetails$ok)
    {
        iset("selected.graphics.subset", gdetails$selected.samples)
        iset("value.x", gdetails$selected.channels[[1]])
        iset("value.y", NULL)
        openGraphics(mode = 1)
    }
}


ecdf.menu <- function(action, window)
{
    graphics.choice <- "ecdf"
    iset("graphics.choice", graphics.choice)
    gdetails <- ask.graphics.details(action, window, allow.multiple.samples = TRUE, channel.selections = "Channel", allow.multiple.channels = FALSE)
    if (gdetails$ok)
    {
        iset("selected.graphics.subset", gdetails$selected.samples)
        iset("value.x", gdetails$selected.channels[[1]])
        iset("value.y", NULL)
        openGraphics(mode = 1)
    }
}

workflow.menu <- function(action, window)
{
   openGraphics(mode = 4)
  
}  

