## Accessor to the internal GUI environment 
guiEnv <- function()
    environment(iflow)[[".guiEnv"]]



iflow <- function()
{
    entries <-
        list(
             ## name, stock.id (prefab icons), label (. signifies mneumonic)
             list("FileMenu",         NULL, "_File"),
             list("DataMenu",         NULL, "_Data"),
             list("GraphicsMenu",     NULL, "_Graphics"),
             list("GateMenu",         NULL, "_Gate"),  
             list("ProbBinMenu",      NULL, "_ProbBin"), 
             list("HelpMenu",         NULL, "_Help"),  

             list("Open",             "gtk-open", "_Open", "<control>O", "Open a file", open.menu),
             list("Load",             NULL, "_Load", "<control>L", "Load a file", load.menu),
             list("SaveAs",           "gtk-save", "Save _As...", "<control>S", "Save to a file", saveAs.menu),
             list("Quit",             "gtk-quit", "_Quit", "<control>Q", "Quit", quit.menu),
             list("Manual",   NULL, "Manual", NULL, "Manual", manual.menu),
       ##      list("About-Filter",     NULL, "About-Filter", NULL, "About-Filter", help.filter.menu),
             list("Transformation",   NULL,  "_Transformation", "<control>T", "Transformation", trans.menu),
             list("SubsetMenu",       NULL,  "_Subset by"),
             list("ByGates",          NULL, "Gate", NULL, "Gate", sub_filter.menu),
             list("ByPhenoData",      NULL, "Sample Covariates", NULL, "Sample Covariates", sub_pdata.menu),
             list("Compensation",     NULL, "Compensation", NULL, "Compensation",compensation.menu),
             list("Normalization",    NULL, "Normalization", "<control>N", "Normalization",normal.menu),
             list("Summary",       NULL, "Summary", NULL, "Summary",stat.menu),

             list("Contour",          NULL, "_Contour Plot",    NULL, "Contour Plot", contour.menu),
             list("ECDF plot",        NULL, "_ECDF Plot",  NULL, "ECDF Plot", ecdf.menu),
             list("Dot plot",         NULL, "_Dot Plot", NULL, "Dot Plot", dotplot.menu),
             list("Histogram",        NULL, "_Histogram", NULL, "Histogram", histogram.menu),
             list("Parallel",         NULL, "_Parallel Coordinates Plot", NULL, "Parallel Coordinates Plot", parallel.menu),
             list("Q-Q plot",         NULL, "_Q-Q Plot", NULL, "Q-Q Plot", qqmath.menu),
             list("Scatter plot matrix", NULL, "Scatter Plot _Matrix", NULL, "Scatter Plot Matrix", splom.menu),
             list("Stacked density plots", NULL, "_Stacked Density Plots", NULL, "Stacked Density Plots", densityplot.menu),
             list("Smooth scatter plot",     NULL, "_Smooth Scatter Plot", NULL, "Smooth Scatter Plot", xyplot.menu),
             list("Timeline plot",    NULL, "_Timeline Plot", NULL, "Timeline Plot", timeLine.menu),
             list("Workflow plot", NULL, "_Workflow Plot", NULL, "Workflow Plot", workflow.menu),

             list("Create",             NULL, "_Create", "<control>C", "Create", filter.menu),
             list("CombineGates",     NULL, "Combine", NULL,  "Combine", combine.menu),
             list("Backgating",     NULL, "_Backgating",  "<control>B",  "Backgating", backgating.menu),
             list("Export Indices",     NULL, "Export _Indices",  "<control>I",  "Export Indices", export.indices.menu),
             
             list("ProbBin",          NULL, "ProbBin", NULL,  "ProbBin", probin.menu),
             list("iFlow.history",    NULL, "iFlow.history", NULL,  "iFlow.history", history.menu),
             list("Graphics",            "BarGraph", NULL, NULL, "Graphics", graphics.menu),
             list("Draw",           "Draw",  NULL, NULL, "Draw", gate.menu)    )
    

    ## register the icons on the tool bar.
    ## it is ok to separate this function in another file.
    registered <- FALSE

    register.stock.icon <- function()
    {
        if (registered)
            return()
        
        item <- list(c("BarGraph", "_Graphics", 0, 0, NULL), c("Filler", "Filler", 0, 0, NULL), c("Draw", "Draw", 0, 0, NULL))
        gtkStockAdd(item)
        
        ## add an icon
        factory <- gtkIconFactoryNew()
        gtkIconFactoryAddDefault(factory)

        imagename.1 <- system.file("icons/BarGraph.gif", package="iFlow")
        if(file.exists(imagename.1)){
            pixbuf <- gdkPixbufNewFromFile(imagename.1)[[1]]  
            transparent <- pixbuf$addAlpha(TRUE, 255, 255, 255)
            icon.set <- gtkIconSetNewFromPixbuf(transparent)
            factory$add("BarGraph", icon.set)
        } else warning("can not load the icon for a plot")
        
        
        ##   imagename.2 <- system.file("icons/Filler.gif", package="iFlow")
        ##  if(file.exists(imagename.2)){
        ##      pixbuf <- gdkPixbufNewFromFile(imagename.2)[[1]]  
        ##      transparent <- pixbuf$addAlpha(TRUE, 255, 255, 255)
        ##      icon.set <- gtkIconSetNewFromPixbuf(transparent)
        ##      factory$add("Filler", icon.set)
        ##   } else warning("can not load the icon for a plot")
        
        
        imagename.3 <- system.file("icons/Pencil.gif", package="iFlow")
        if(file.exists(imagename.3)){
            pixbuf <- gdkPixbufNewFromFile(imagename.3)[[1]]  
            transparent <- pixbuf$addAlpha(TRUE, 255, 255, 255)
            icon.set <- gtkIconSetNewFromPixbuf(transparent)
            factory$add("Draw", icon.set)
        } else warning("can not load the icon for a plot")
        
        
        TRUE
    }
    
    ## add the icon to the tool bar.
    registered <- register.stock.icon() 

    ## create a window
    window <- gtkWindowNew("toplevel", FALSE)
    window$setDefaultSize(800, 600)
    window$setTitle("iFlow")

    ## add a table layout
    table <- gtkTableNew(3, 6, homogeneous = FALSE)
    window$add(table)

    agroup <- gtkActionGroupNew("iFlowAtions")

    ## add actions to group with the window widget passed to callbacks
    agroup$addActions(entries, window)

    ## create a UI manager to read in menus specified in XML
    manager <- gtkUIManagerNew()

    window$setData("ui-manager", manager)
    manager$insertActionGroup(agroup, 0)

    window$addAccelGroup(manager$getAccelGroup())

    ## Define some XML
    uistr <- "
<ui>
  <menubar name='MenuBar'>
    <menu action='FileMenu'>
      <menuitem action='Open'/>
      <menuitem action='Load'/>
      <menuitem action='SaveAs'/>
      <separator/>
      <menuitem action='Quit'/>
    </menu>
    <menu action='DataMenu'>
        <menuitem action='Transformation'/>
        <menu action='SubsetMenu'>
            <menuitem action = 'ByGates'/>
            <menuitem action = 'ByPhenoData'/>
         </menu>
        <menuitem action='Compensation'/>
        <menuitem action='Normalization'/>
        <menuitem action='Summary'/>
    </menu>
      <menu action='GraphicsMenu'>
         <menuitem action='Contour'/>
         <menuitem action='ECDF plot'/>
         <menuitem action='Dot plot'/>
         <menuitem action='Histogram'/>
         <menuitem action='Parallel'/>
         <menuitem action='Q-Q plot'/>
         <menuitem action='Scatter plot matrix'/>
         <menuitem action='Smooth scatter plot'/>
         <menuitem action='Stacked density plots'/>
         <menuitem action='Timeline plot'/>
         <menuitem action='Workflow plot'/>
    </menu>
 <menu action='GateMenu'>
         <menuitem action='Create'/>
         <menuitem action='CombineGates'/>
         <menuitem action='Backgating'/>
         <menuitem action='Export Indices'/>
         </menu>
<menu action='ProbBinMenu'>
      <menuitem action= 'ProbBin'/>
 </menu>
 <menu action='HelpMenu'>
      <menuitem action='Manual'/>
      <menuitem action='iFlow.history'/>
    </menu>
  </menubar>
  <toolbar  name='ToolBar'>
    <toolitem action='Open'/>
    <toolitem action='SaveAs'/>
    <toolitem action='Quit'/>
    <separator action = 'Sep1'/>
    <toolitem action='Graphics'/>
    <toolitem action='Draw'/>
  </toolbar>
</ui>
"

    manager$addUiFromString(uistr)
    menubar <- manager$getWidget("/MenuBar")
    menubar$show()    # location, layout behavior, padding
    table$attach(menubar, 0, 1, 0, 1, c("expand", "fill"), 0, 0, 0)

    bar <- manager$getWidget("/ToolBar")
    bar$setTooltips(TRUE)
    bar$show()
    table$attach(bar, 0, 1, 1, 2, c("expand", "fill"), 0, 0, 0)
    
    #######################################################################################
    ### outline of control panel and data panel on main window
    HPaned <- gtkHPanedNew()
    HPaned$setBorderWidth(5)
    
    #########################################################################################
    ### outline for data section and filter list on a control panel
    VPaned <- gtkVPanedNew()
    HPaned$add(VPaned)
    
    #########################################################################################
    ### Data Selection
    sw <- gtkScrolledWindowNew(NULL, NULL)
    sw$setShadowType("etched-in")
    sw$setPolicy("automatic", "automatic")

    vbox <- gtkVBox(TRUE, 0)
    vbox$setSizeRequest(150, 275)
    vbox$packStart(sw, TRUE, TRUE, 0) # for data

    VPaned$add(vbox)
    
    ## create tree model for data selection
    iset("model", create.model())
    ## create tree view
    treeview <- gtkTreeViewNewWithModel(iget("model"))
    treeview$setRulesHint(TRUE)
    treeview$getSelection()$setMode("single")
    treeview$setHeadersClickable(FALSE)
    selected <- treeview$getSelection()
    iset("dataSetTreeView", treeview)

    add.columns(treeview)
    sw$add(treeview)

    ## to make selected data active
    gSignalConnect(selected, "changed", dataChoice, c(treeview, action))
    ## to show the transformation info in a gate/transformation view
    gSignalConnect(selected, "changed", trans.cb, treeview)
      
    #########################################################################################
    ### Gate List
    sw1 <- gtkScrolledWindowNew(NULL, NULL)
    sw1$setShadowType("etched-in")
    sw1$setPolicy("automatic", "automatic")

    vbox1 <- gtkVBox(TRUE, 0)
    vbox1$packStart(sw1, TRUE, TRUE, 0) 
    
    Gate.list <- list()
      
    iset("fcon", rGtkDataFrame(data.frame(cols = names(.guiEnv$Gate.list))))
    iset("filterview", gtkTreeView(iget("fcon")))
    fcol <- gtkTreeViewColumn("Gate List", gtkCellRendererText(), text = 0)
    iget("filterview")$insertColumn(fcol, 0)
    iget("filterview")$getSelection()$setMode("single")
    selfilter <- iget("filterview")$getSelection()
    
    ## for selected filter information
    gSignalConnect(selfilter, "changed", filter.cb, list(iget("fcon"), iget("filterview")))
    ## for pop up menu
    view <- iget("filterview")
    gSignalConnect(view, "button-press-event", button.cb, 
                              c(view, action, "filter"))
    
    sw1$add(iget("filterview"))    
    VPaned$add(vbox1)
    
    #################################################################################################
    ### for drag and drop
    # for the properties of drag & drop
    TARGET <- c(string = 0)
    targetentries <-
        list(
          c("STRING", 0, TARGET[["string"]]),
          c("text/plain", 0, TARGET[["string"]]) 
     )
    
    gtkDragSourceSet(iget("filterview"), "button1-mask", targetentries, "copy")
    gtkDragDestSet(treeview, "all", targetentries, "copy")
    
    gSignalConnect(iget("filterview"), "drag-data-get", drag_get, NULL)
    gSignalConnect(treeview, "drag-data-received", drop_get, NULL)
   
    
     
    #######################################################################################
    ### data panel
    ## create a notebook 

    vbox <- gtkVBox(FALSE, 0)
    HPaned$add(vbox)
    iset("notebook", gtkNotebook())          
    iget("notebook")$setTabPos("top")
    vbox$packStart(iget("notebook"), TRUE, TRUE, 0)
  

    ## create data information view in information page
    
    iset("data.info.list", NULL) 
    iset("datacon", rGtkDataFrame(data.frame(cols = iget("data.info.list"))))
    iset("dataview", gtkTreeView(iget("datacon")))
    
    datacol <- gtkTreeViewColumn("Data View", gtkCellRendererText(), text = 0)
    iget("dataview")$insertColumn(datacol, 0)
    iget("dataview")$getSelection()$setMode("none")

  
    ## create information for the filter and transformation view in the information page
    
    infolist <- NULL
    iset("infolist", infolist) 

    iset("incon", rGtkDataFrame(data.frame(cols = iget("infolist"))))
    iset("infoview", gtkTreeView(iget("incon")))
    incol <- gtkTreeViewColumn("Transformation View", gtkCellRendererText(), text = 0)
    iget("infoview")$insertColumn(incol, 0)
    iget("infoview")$getSelection()$setMode("none")
    
    gatelist <- NULL
    iset("gatelist", gatelist) 

    iset("gatecon", rGtkDataFrame(data.frame(cols = iget("gatelist"))))
    iset("gateview", gtkTreeView(iget("gatecon")))
    incol <- gtkTreeViewColumn("Gate View", gtkCellRendererText(), text = 0)
    iget("gateview")$insertColumn(incol, 0)
    iget("gateview")$getSelection()$setMode("none")

    showDataInfo(iget("dataview"), iget("filterview"), iget("transview"), iget("infoview"), iget("gateview"))
    
    #######################################################################################################
    ##scrolled.window$setShadowType("in") # add scrolled window below toolbar and fill the remaining space
    table$attach(HPaned, 0, 1, 2, 3, c("expand", "fill"), c("expand", "fill"), 0, 0)

    ## set up global variables 
    initializeEnv()
    window$showAll()
}


