
load.object <- function(objname, df = base::get(objname, pos = .GlobalEnv))
{
    if (is(df, "flowSet"))
    {
        wfname <- tail(make.unique(c(names(iget("workflow.list")), objname)), 1)
        entryname <- 
            if (wfname == objname) objname
            else sprintf("%s [%s]", wfname, objname)
        wf <- workFlow(df, objname)
        .guiEnv[["workflow.list"]][[entryname]] <- wf
        node.path <- add.node(entryname, parent = FALSE)
        updateGlobalNodelist(node.path, identifier(wf[["base view"]]))
        TRUE
    }
    else FALSE
}


load.menu <- function(action, window)
{
    ## dialog to select file
    objs <- ls(.GlobalEnv)
    objs <- objs[sapply(objs, function(x) is(get(x, .GlobalEnv), "flowSet"))]
    if (length(objs))
    {
        dialog <- gtkDialogNewWithButtons(if (length(objs) == 1) "Select flowset" else "Select flowset(s)",
                                          window, "destroy-with-parent", 
                                          "gtk-ok", GtkResponseType["ok"],
                                          "gtk-cancel", GtkResponseType["cancel"])
        dialog$setSizeRequest(300, -1)  
        ## FIXME: should diable OK button when no items selected
        checkBoxes <- sapply(objs, function(x) {
            check <- gtkCheckButton(x) 
            dialog[["vbox"]]$packStart(check, TRUE, TRUE, 0)
            check	
        })
        if (dialog$run() == GtkResponseType["ok"])
        { 
            act <- sapply(checkBoxes, function(x) x$getActive())
            if (any(act))
            {
                tryCatch(sapply(objs[act], load.object), 
                         error = function(e) {
                             m <- paste("Error: ", conditionMessage(e))
                             error.handler(m)
                         },  
                         warning = function(e) {
                             m <- paste("Warning: ", conditionMessage(e))
                             warning.handler(m)
                         })
            }
        }
        dialog$destroy()
    }
    else{
        ## warning that there are no sets to load
        show.message("No flowsets found in global workspace.")
    }
}

loadFiles <- function(filenames)
{
    
    ## We want to notify user if no flowsets were ever added.  We do
    ## this via a flag
    foundSomething <- FALSE

    if (length(filenames) == 0)
        return()
    ex <- sapply(strsplit(filenames, "\\."), tail, 1)

    processRdaFile <- function(i) 
    {
        tmp.env <- new.env()
        load(filenames[i], tmp.env)
        status <-
            sapply(ls(tmp.env), function(s) {
                load.object(s, df = tmp.env[[s]])
            })
        any(status)
    }

    RDAFILES <- ex %in% c("rda", "rdata")
    if (any(RDAFILES)) {
        ## load rda files
        foundSomething <- any(sapply(which(RDAFILES), processRdaFile))
    }
    if (all(RDAFILES)) return()

    ## Rest, which are all hopefully fcs files, but we will check!
    filenames <- filenames[!RDAFILES]
    FCSFILES <- isFCSfile(filenames)

    if (any(!FCSFILES)) show.message("Not all files were RDA or FCS files.")
    
    if (any(FCSFILES))
    {
        foundSomething <- TRUE
        filenames <- filenames[FCSFILES]
        df <- read.flowSet(filenames)
        newName <- getNewName("Name of flowset", "Anonymous")
        load.object(newName, df = df)
    }
    if (!foundSomething)
        show.message("No valid flowSets or FCS files were found.")
}

