
openGraphics <-
    function(..., mode)
{
    ## event handler to save an image
    saveimage.menu <- function(action, win) {
        dialog <- gtkFileChooserDialog("Enter a file name", win, "save",
                                       "gtk-cancel", GtkResponseType["cancel"],
                                       "gtk-save", GtkResponseType["accept"])
        if (dialog$run() == GtkResponseType["accept"]) {
            filename <- dialog$getFilename()
            dev.copy(png, filename)
            dev.off()
        }
        dialog$destroy()
    }
    
    ## event handler to print an image
    print.menu <- function(action, win){
      show.message("fixing errors....")
      ##  printing()
    }

    ## event handler to close the plot
    quit.menu <- function(action, win){
        win$destroy()
    }

    ## setup the plot dialog window
    entries <- list(
                    ## name, stock.id (prefab icons), label (. signifies mneumonic)
                   list("SaveMenu", NULL, "_Save"),
                   list("PrintMenu", NULL, "_Print"),
                   list("QuitMenu", NULL, "_Quit"),
                    list("SaveAs", "gtk-save", "SaveAsImage...", NULL, "Save to a file",
                         saveimage.menu),
                    list("Print", "gtk-print", "_Print", "<control>P", "Print the current graph",
                         print.menu), 
                    list("Quit", "gtk-quit", "_Quit", "<control>Q", "Quit", quit.menu))
    
    win <- gtkWindow(show = FALSE)
    win$setTitle("Graphics")
    
    ## add a table layout
    table <- gtkTableNew(2, 4, FALSE)
    win$add(table)
    agroup <- gtkActionGroupNew("graphicsAtions")

    ## add actions to group with the window widget passed to callbacks
    agroup$addActions(entries, win)

    ## create a UI manager to read in menus specified in XML
    manager <- gtkUIManagerNew()

    win$setData("ui-manager", manager)
    manager$insertActionGroup(agroup, 0)

    win$addAccelGroup(manager$getAccelGroup())
    
    ## Define some XML
    uistr <- paste(
                   "<ui>",
                   "  <toolbar  name='ToolBar'>",
                   "    <toolitem action='SaveAs'/>",
                   "    <toolitem action='Print'/>",
                   "    <toolitem action='Quit'/>", 
                   "    <separator action='Sep1'/>",
                   "  </toolbar>",
                   "</ui>", sep="\n")

    manager$addUiFromString(uistr)

    bar <- manager$getWidget("/ToolBar")
    bar$setTooltips(TRUE)
    bar$show()
    table$attach(bar, 0, 1, 1, 2, c("expand", "fill"), 0, 0, 0)
    
    ##### for drawing plots ######
    HPaned <- gtkHPanedNew()
    HPaned$setBorderWidth(5)
    
    graphics<- gtkDrawingArea()
    dev_list <- list()
    dev_list[[1]] <- graphics
    #sz <- 600
   
   
    sw <- gtkScrolledWindowNew(NULL, NULL)
    sw$setShadowType("etched-in")
    sw$setPolicy("automatic", "automatic")
    sw$addWithViewport (dev_list[[1]])
    
    #sw <- make.scrolled_window(dev_list[[1]])
    
    hbox <- gtkHBox(TRUE, 0)
    hbox$packStart(sw, TRUE, TRUE, 0) # for data

    HPaned$add(hbox)
    	
    if(mode == 4)
    {
        if (!isAnyActive()) 
        {
           show.message("No active dataset")
           return()
        }
    
    
       VPaned <- gtkVPanedNew()
       HPaned$add(VPaned)  
        
       hbox <- gtkVBox(FALSE, 10)
       VPaned$add(hbox)
       
       frame <- gtkFrame("Covariate & Samples")
       hbox$packStart(frame, TRUE, TRUE, 1)
 
       button <- gtkButtonNewWithLabel("Draw", show = TRUE)  
       hbox$packStart(button, FALSE, FALSE, 10)
 
    
        vbox <- gtkHBox(FALSE, 10)
        frame$add(vbox)
     
        pheno <- colnames(activeFlowset())
        samples <- rownames(pData(activeFlowset()))
                       
         prdf <- rGtkDataFrame(data.frame(cols=pheno))
         pview <- gtkTreeView(prdf)
         pcol <- gtkTreeViewColumn("Select : X", gtkCellRendererText(), text = 0)
         pview$insertColumn(pcol, 0)
         pview$getSelection()$setMode("single")
         sel1 <- pview$getSelection()
    
         sw2 <- make.scrolled_window(pview)
    
         vbox$packStart(sw2, TRUE, TRUE, 0)          
         gating <- .guiEnv$Gate.list
         
         ## not related to the gui ######################### 
         ## but necessary to design the gui for channel view
         cur_node <- iget("cur_node")
         wf <- activeWorkflow()
             	            
	 parent_node <- parent(wf[[cur_node]])@name
	   ## find index of a selected filter in a gate list
          actionId <-action(wf[[cur_node]])@name
          ind <- gate.action(actionId)
       
        if(length(gating[[ind]]@parameters) != 1)
        {	
           view <- gtkTreeView(prdf)
           col <- gtkTreeViewColumn("Select : Y", gtkCellRendererText(), text = 0)
           view$insertColumn(col, 0)
           view$getSelection()$setMode("single")
           sel2 <- view$getSelection()
    
           sw1 <- make.scrolled_window(view)
    
           vbox$packStart(sw1, TRUE, TRUE, 0)
         
        }
        
        ## preselection for lymphFilter
 #       if(class(gating[[ind]]) == "lymphFilter")
 #       {
 #          view2 <- gtkTreeView(prdf)
 ##          col <- gtkTreeViewColumn("Preselection", gtkCellRendererText(), text = 0)
 #          view2$insertColumn(col, 0)
 ##          view2$getSelection()$setMode("single")
 #          sel3 <- view2$getSelection()
    
 #          sw3<- make.scrolled_window(view2)
 #   
  #         vbox$packStart(sw3, TRUE, TRUE, 0)	
       	
  #        }
         ## for samples
         srdf <- rGtkDataFrame(data.frame(cols=samples))
         sview <- gtkTreeView(srdf)
         scol <- gtkTreeViewColumn("Samples", gtkCellRendererText(), text = 0)
         sview$insertColumn(scol, 0)
         sview$getSelection()$setMode("multiple")
         sel4 <- sview$getSelection()
    
         sw4 <- make.scrolled_window(sview)
    
         vbox$packStart(sw4, TRUE, TRUE, 0)  	
        
   
    }
     
    table$attach(HPaned, 0, 1, 2, 3, c("expand", "fill"), c("expand", "fill"), 0, 0)
    win$setDefaultSize(800, 600)
    win$showAll()

    ######## embedd the output in a Cairo device
    asCairoDevice(dev_list[[1]])

    ## regular plots
    if(mode == 1)                 
    {
        tryCatch(graph.selection(...),
                 error = function(e) {
                     m <- paste("Error:", conditionMessage(e))
                     error.handler(m)
                 }, 
                 warning = function(e) {
                     m <- paste("Warning:", conditionMessage(e))
                     warning.handler(m)
                 })
    }
    else if (mode == 4){
    	     ######### graphics for backgating ######################################
       draw.backgating.graph( button, pview, title, view, sview)
       
    }

  else if(mode == 2){
  	  ## draw graphics for a gate in active data.
	 draw.gating.graphics()
}

    ## for rectangle gate...
    else if(mode == 3){
          tryCatch(
          { 
            x <- activeFlowset()
            
            form <- sprintf("`%s` ~ `%s`", value.y, value.x) 
          #  fcall <- sprintf("plot(xyplot(%s, data=dt))", form)
          #  evaluateCommand(fcall)
             fcall <- sprintf("xyplot(%s, data=dt)", form)
          plot(eval(parse(text = fcall))) }, 
          
           error = function(e) {
                m <- paste("Error was produced.", conditionMessage(e))
                error.handler(m)
            }, 
            warning = function(e) {
               m <- paste("Waring was produced.", conditionMessage(e))
               warning.handler(m)
            }
    
          )   
             pop <- 1
            iset("pop", pop)
           
    }
    
    ## for hierarchy of workflow object...
    else if(mode == 4){
        tryCatch(
        { plot(activeWorkflow()) }, 
         error = function(e) {
                m <- paste("Error was produced.", conditionMessage(e))
                error.handler(m)
            }, 
            warning = function(e) {
               m <- paste("Waring was produced.", conditionMessage(e))
               warning.handler(m)
            }
          )  
    }
    ## for probin
    else{
               values <- c(iget("value.x"), iget("value.y"))
               resCtrl <- iget("variable.y")
               stat <- iget("variable.x")
               fset <- iget("dfrm")
               if(length(fset) == 1)
                   plotBins(resCtrl,fset[[1]],channels=values,title="Binned control data")
               else{
                   title <- paste("Comparison", identifier(fset[[1]]), "&", identifier(fset[[2]]))
                   plotBins(resCtrl,data=fset[[2]],channels=values,title= title,
                            residuals=stat$residuals[2,],shadeFactor=0.7) 
      
       }
    }
    
}




## call a lattice densityplot 
densPlot <- function(channels){
    channels <- paste(sprintf("`%s`", channels), collapse=" + ", sep="")
    fcall <- sprintf("plot(densityplot(~ %s, data=dt))", channels)
    print("Creating densityplot...")
    evaluateCommand(fcall)  
    pop <- 1
    iset("pop", pop)
  
}

## call a lattice xyplot
xyPlot <- function(x, y){
    fcall <- sprintf("plot(xyplot(`%s`~`%s`, data=dt))", y, x)
    print("Creating xyplot...")
    evaluateCommand(fcall) 
    pop <- 1
    iset("pop", pop)
  
}
