
getNewName <- function(title, initial.name)
{
    dialog <- gtkDialogNewWithButtons(title, iget("window"), "destroy-with-parent", 
                                      "gtk-ok", GtkResponseType["ok"], "gtk-cancel",
                                      GtkResponseType["cancel"]) 
    hbox <- gtkHBoxNew(TRUE, 0)
    fname <- gtkLabelNew("Input a name for the new data node")
    text <- initial.name
    entry <- gtkEntryNew()
    entry$setText(text)
    entry$"has-focus" <- TRUE
    ## FIXME: Would be nice if hitting the return key would be equivalent to prressing
    ## OK with the mouse...
    hbox$packStart(fname, FALSE, FALSE, 0)
    hbox$packStart(entry, FALSE, FALSE, 0)
    dialog[["vbox"]]$packStart(hbox, FALSE, FALSE, 0)
    if (dialog$run() == GtkResponseType["ok"]){
        text <- entry$getText()
    }      
    dialog$destroy()
    text
}


show.message <- function(m){
    dialog.msg <- gtkMessageDialogNew(NULL, c("modal", "destroy-with-parent"), "info", "ok", m)
    dialog.msg$run()
    dialog.msg$destroy()
}


populateDataView <- function(s = character(0))
{
    iget("datacon")$setFrame(data.frame(cols = s))
}

activeFlowset <- function()
{
    with(.guiEnv,
     {
         wf <- workflow.list[[active.workflow]]
         if (is.null(wf)) new("flowSet") ## empty flowSet
         else Data(get(activeNodeID(), wf))
     })
}

isAnyActive <- function()
{
    length(activeFlowset()) > 0
}

activeWorkflow <- function()
{
    with(.guiEnv,
         workflow.list[[active.workflow]])
}

updateGlobalNodelist <- function(i, node.id)
{
    .guiEnv$global.node.list[[ paste(i, collapse = "_") ]] <- node.id
}


activeNodeID <- function()
{
    i <- getSelectedIndices(iget("dataSetTreeView"))
    if (length(i) == 0) NULL 
    else .guiEnv$global.node.list[[ paste(i, collapse = "_") ]]
}

activeNodeName <- function()
{
    nm <- flowCore:::nameIdMapping(activeWorkflow())[activeNodeID()]
    if (nm == "base view") activeWorkflow()@name
    else nm
}


getSelectedIndices <- function(x)
{
    ## x should be a "GtkTreeView", as far as we can tell, but not going to check.
    unlist(lapply(x$getSelection()$getSelectedRows()$retval, gtkTreePathGetIndices)) + 1L
}

setSelectedIndices <- function(x, w)
{
    ## x should be a "GtkTreeView"
    sel <- x$getSelection()
    sel$unselectAll()
    lapply(w, function(i) sel$selectPath(gtkTreePathNewFromIndices(i-1L)))
    invisible()
}

activePhenoSubset <- function()
{
    if (iget("active.workflow") == "") NULL
    else getSelectedIndices(iget("phenodata.tree.view"))
    
}



add.node <- function(node.name, parent = TRUE)
{
    model <- iget("model")
    parent.iter <-
        if (parent)
            iget("dataSetTreeView")$getSelection()$getSelected()$iter
        else NULL
    iter <- model$append(parent.iter)$iter
    model$set(iter, 0, node.name)
    model$getPath(iter)$GetIndices() + 1
}


addSubNode <- function(wf.action, suffix = "")
{
    
    old.nodes <- nodes(activeWorkflow())
    add(activeWorkflow(), wf.action,
        parent = activeNodeID(),
        name = getNewName("Enter a name for the flow set",
                          sprintf("%s_%s", activeNodeName(), suffix)))
    new.nodes <- setdiff(nodes(activeWorkflow()), old.nodes)
    new.names <- flowCore:::nameIdMapping(activeWorkflow())[new.nodes]
    sapply(seq_along(new.nodes),
           function(i) {
               node.path <- add.node(new.names[i])
               updateGlobalNodelist(node.path, new.nodes[i])
            })
  
   
    ## for the transformation info view       
    if(iget("trans_mode") == TRUE)
    {
    	method <- iget("tmethod")
        method[[1]] <- c(method[[1]], new.names[[1]])
        method[[2]] <- c(method[[2]], new.nodes)
        iset("tmethod", method)
     } 
}

make.treeview <- function(object, title, choice)
{
  ## object is a vector, choice is either "single" or "multiple"
    rdf <- rGtkDataFrame(data.frame(cols=object))
    view <- gtkTreeView(rdf)
    col <- gtkTreeViewColumn(title, gtkCellRendererText(), text = 0)
    view$insertColumn(col, 0)
    view$getSelection()$setMode(choice)
    view
}  
 
make.scrolled_window <- function(x)
{
  ## x should be a "GtkTreeView"
    sw <- gtkScrolledWindowNew(NULL, NULL)
    sw$setShadowType("etched-in")
    sw$setPolicy("automatic", "automatic")
    sw$add(x)
    sw
}  


